/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.util.List;
import javax.persistence.Query;
import javax.swing.ActionMap;
import javax.swing.text.DefaultEditorKit;
import nl.cloudfarming.client.model.CommonModelService;
import nl.cloudfarming.client.model.Culture;
import nl.cloudfarming.client.model.DataProvider;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.FieldPlan;
import nl.cloudfarming.client.model.Operation;
import nl.cloudfarming.client.model.Task;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;
import org.openide.util.lookup.ServiceProviders;

/**
 * Persistence service for Tactical planning
 *
 * @author Timon Veenstra
 */
@ServiceProviders(value = {
    @ServiceProvider(service = DataProvider.class),
    @ServiceProvider(service = TacticalPlanningService.class)
})
public class TacticalPlanningService extends CommonModelService {

    private final FieldPlanRootNode rootNode = new FieldPlanRootNode();

    @Override
    protected String getModuleName() {
        return TacticalPlanningModule.MODULE_NAME;
    }

    /**
     * Updates a Operation. 
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    public void updateOperation(Operation toBeMerged) {
        startTransaction();
        if (toBeMerged.getId() == 0) {
            throw new IllegalArgumentException("The OPeration cannot be updated because it is not persisted yet."); 
        } else {
            toBeMerged = getEntityManager().merge(toBeMerged);
        }
        commit();
    }

    /**
     * updates a task
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    public void updateTask(Task toBeMerged) {
        startTransaction();
        if (toBeMerged.getId() == 0) {
            throw new IllegalArgumentException("The OPeration cannot be updated because it is not persisted yet."); 
        } else {
            toBeMerged = getEntityManager().merge(toBeMerged);
        }
        commit();
    }

    /**
     * updates a Culture
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    public void updateCulture(Culture toBeMerged) {
        startTransaction();
        if (toBeMerged.getId() == 0) {
            throw new IllegalArgumentException("The OPeration cannot be updated because it is not persisted yet."); 
        } else {
            toBeMerged = getEntityManager().merge(toBeMerged);
        }
        commit();
    }

    /**
     * Creates an Operation within a transaction. Starts a transaction if it is not already started.
     * @param operation used as parameter placeholder, use return value
     * @return 
     */
    public Operation createOperation(Operation.Builder operationBuilder) {
        Operation o = operationBuilder.build();
        startTransaction();
        getEntityManager().persist(o);
        commit();
        return o;
    }

    /**
     * Creates an Task within a transaction. Starts a transaction if it is not already started.
     * @param Task used as parameter placeholder, use return value
     * @return 
     */
    public Task createTask(Task.Builder taskBuilder) {
        Task t = taskBuilder.build();
        startTransaction();
        getEntityManager().persist(t);
        commit();
        return t;
    }

    /**
     * Creates an Culture within a transaction. Starts a transaction if it is not already started.
     * @param Culture used as parameter placeholder, use return value
     * @return 
     */
    public Culture createCulture(Culture.Builder cultureBuilder) {
        Culture c = cultureBuilder.build();
        startTransaction();
        getEntityManager().persist(c);
        getEntityManager().getTransaction().commit();
        return c;
    }

    /**
     * Creates an FieldPlan within a transaction. Starts a transaction if it is not already started.
     * @param year for which Fieldplan is created
     */
    public FieldPlan createFieldPlanForYear(int year) {
        FieldPlan fieldPlan = new FieldPlan();
        fieldPlan.setPlanYear(year);
        fieldPlan.setName(NbBundle.getMessage(this.getClass(), "field_plan.new_field_plan_action.fieldplan_name", year + ""));
        persist(fieldPlan);
        return fieldPlan;
    }

    /**
     * Creates an Field within a transaction. Starts a transaction if it is not already started.
     * @param Builder of field
     */
    public Field createField(Field.Builder builder) {
        Field field = builder.build();
        persistInTransaction(field);
        return field;
    }

    /**
     * find all fieldplans
     *
     * @return List of FieldPlans
     */
    public List<FieldPlan> findFieldPlans() {
        return getEntityManager().createNamedQuery(FieldPlan.Q_FIND_ALL_FIELD_PLANS).getResultList();
    }

    /**
     * Finds all available Cultures
     * @return List of cultures
     */
    public List<Culture> findAllCultures() {
        startTransaction();
        Query query = getEntityManager().createNamedQuery(Culture.Q_FIND_ALL_CULTURES);
        return query.getResultList();
    }

    /**
     * Finds the culture by the suplied id
     * @param id: the id of the Culture which must be found
     * @return culture
     */
    Culture findCultureById(Long id) {
        startTransaction();
        return getEntityManager().find(Culture.class, id);
    }

    /**
     * Finds al the available Operations
     * @return List of Operation
     */
    public List<Operation> findAllOperations() {
        startTransaction();
        Query query = getEntityManager().createNamedQuery("findAllOperations");
        return query.getResultList();
    }
    
    /**
     * Finds an Operation by its id
     * @param id: the id of the Operation which has to be found
     * @return if found Operation otherwise null 
     */
    Operation findOperationById(long id) {
        startTransaction();
        return getEntityManager().find(Operation.class, id);
    }

    /**
     * Find all the available Tasks
     * @return List Task
     */
    public List<Task> findAllTasks() {
        startTransaction();
        Query query = getEntityManager().createNamedQuery(Task.Q_FIND_ALL_TASKS);
        return query.getResultList();
    }

    /**
     * Find all the available Fields
     * @return List Field
     */
    public List<Field> findAllFields() {
        startTransaction();
        Query query = getEntityManager().createNamedQuery(Field.Q_FIND_ALL_FIELDS);
        return query.getResultList();
    }

    /**
     * Finds Task by the supplied id
     * @param id: the id of the Task which has to be found
     * @return if found Task else null
     */
    Task findTaskById(Long id) {
        startTransaction();
        return getEntityManager().find(Task.class, id);
    }

    /**
     * Finds all the operations which are connected to the culture via task
     * @param id: the id of the Culture is connected
     * @return List Operation
     */
    public List<Task> findTaskByCultureId(Long cultureId) {
        startTransaction();
        Query query = getEntityManager().createNamedQuery(Task.Q_FIND_TASK_BY_CULTURE_ID);
        query.setParameter("id", cultureId);
        return query.getResultList();
    }
    
    /**
     * removes Culture from fieldplan. Checks if the culture which will be removed is managed by the enttity manager
     * @param culture which will be removed.
     */
    public void removeCultureFromFieldPlan(Culture culture) {
        startTransaction();
        if (getEntityManager().contains(culture)) {
            getEntityManager().remove(culture);
        } else{
            getEntityManager().remove(getEntityManager().merge(culture));
        }
        if(culture.getFieldplan() != null && culture.getFieldplan().getCultures() != null){    
            culture.getFieldplan().getCultures().remove(culture);
        }
        getEntityManager().getTransaction().commit();
    }

    @Override
    protected void initExplorer() {
        ExplorerManager manager = new ExplorerManager();
        manager.setRootContext(rootNode);
        manager.getRootContext().setDisplayName(NbBundle.getMessage(this.getClass(), "planning_explorer.rootnode"));

        ActionMap map = new ActionMap();
        map.put(DefaultEditorKit.copyAction, ExplorerUtils.actionCopy(manager));
        map.put(DefaultEditorKit.cutAction, ExplorerUtils.actionCut(manager));
        map.put(DefaultEditorKit.pasteAction, ExplorerUtils.actionPaste(manager));
        map.put("delete", ExplorerUtils.actionDelete(manager, true)); // or false

        setExplorerManager(manager, map);
    }
}
