/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.wizard;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.Date;
import javax.swing.JComponent;
import nl.cloudfarming.client.model.Culture;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.Operation;
import nl.cloudfarming.client.model.Task;
import nl.cloudfarming.client.planning.PlanAction;
import nl.cloudfarming.client.planning.PlanEventKey;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningService;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventKey;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

// An example action demonstrating how the wizard could be called from within
// your code. You can move the code below wherever you need, or register an action:
// @ActionID(category="...", id="nl.cloudfarming.client.planning.tactical.PlanningWizardAction")
// @ActionRegistration(displayName="Open Planning Wizard")
// @ActionReference(path="Menu/Tools", position=...)
public final class PlanningWizardAction extends PlanAction implements ActionListener {

    private TacticalPlanningService service;
    private WizardDescriptor.Panel[] panels;
    private final String DISPLAY_NAME = NbBundle.getMessage(this.getClass(), "field_plan.plan_wizard_action.name");
    private final String TITLE = NbBundle.getMessage(this.getClass(), "field_plan.plan_wizard_action.title");
    private final PlanningEventProducer planningEventProducer = new PlanningEventProducer();

    public PlanningWizardAction() {
        putValue(NAME, DISPLAY_NAME);
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        WizardDescriptor wizardDescriptor = new WizardDescriptor(getPanels());
        // {0} will be replaced by WizardDesriptor.Panel.getComponent().getName()
        wizardDescriptor.setTitleFormat(new MessageFormat("{0}"));
        wizardDescriptor.setTitle(TITLE);
        Dialog dialog = DialogDisplayer.getDefault().createDialog(wizardDescriptor);
        dialog.setVisible(true);
        dialog.toFront();
        boolean cancelled = wizardDescriptor.getValue() != WizardDescriptor.FINISH_OPTION;
        if (!cancelled) {
            Operation operation = saveWizardContent(wizardDescriptor);
            planningEventProducer.triggerEvent(new OperationPlanEvent(operation, PlanEventKey.NEW_OPERATION));
        }
    }

    protected Operation saveWizardContent(WizardDescriptor wizardDescriptor) {
        String operationName = NbBundle.getMessage(this.getClass(), "planning.wizzard.action.spraying.name");
        String[][] agents = (String[][]) wizardDescriptor.getProperty(PlanningWizardPanel2.AGENTS);
        Date planDate = (Date) wizardDescriptor.getProperty(PlanningWizardPanel1.PLANDATE);

        Task.Builder taskBuilder = new Task.Builder();
        Culture culture = Utilities.actionsGlobalContext().lookup(Culture.class);
//      for(String[] agent : agents){
//        //TODO make agents
//      }

        if (culture != null) {
            Operation operation = null;
            for (Field field : culture.getFields()) {
                Operation.Builder operationBuilder = new Operation.Builder();
                operationBuilder.setName(operationName).setField(field);
                operation = getService().createOperation(operationBuilder);

                taskBuilder.setPlanDate(planDate).setOperation(operation);

                getService().createTask(taskBuilder);
            }
            return operation;
        }
        return null;

    }

    /**
     * Initialize panels representing individual wizard's steps and sets
     * various properties for them influencing wizard appearance.
     */
    private WizardDescriptor.Panel[] getPanels() {
        if (panels == null) {
            panels = new WizardDescriptor.Panel[]{
                new PlanningWizardPanel1(),
                new PlanningWizardPanel2(),
                new PlanningWizardPanel3()
            };
            String[] steps = new String[panels.length];
            for (int i = 0; i < panels.length; i++) {
                Component c = panels[i].getComponent();
                // Default step name to component name of panel. Mainly useful
                // for getting the name of the target chooser to appear in the
                // list of steps.
                steps[i] = c.getName();
                if (c instanceof JComponent) { // assume Swing components
                    JComponent jc = (JComponent) c;
                    // Sets step number of a component
                    // TODO if using org.openide.dialogs >= 7.8, can use WizardDescriptor.PROP_*:
                    jc.putClientProperty("WizardPanel_contentSelectedIndex", Integer.valueOf(i));
                    // Sets steps names for a panel
                    jc.putClientProperty("WizardPanel_contentData", steps);
                    // Turn on subtitle creation on each step
                    jc.putClientProperty("WizardPanel_autoWizardStyle", Boolean.TRUE);
                    // Show steps on the left side with the image on the background
                    jc.putClientProperty("WizardPanel_contentDisplayed", Boolean.TRUE);
                    // Turn on numbering of all steps
                    jc.putClientProperty("WizardPanel_contentNumbered", Boolean.TRUE);
                }
            }
        }
        return panels;
    }

    protected TacticalPlanningService getService() {
        if (service == null) {
            service = new TacticalPlanningService();
        }
        return service;
    }

    public void setService(TacticalPlanningService service) {
        this.service = service;
    }

    private class OperationPlanEvent implements GuiEvent<Operation> {

        private final Operation operation;
        private final PlanEventKey key;

        /**
         * Constructor
         * @param fieldPlan: the fieldplan for which the event is triggered
         * @param key: the event key. Which action has been done to trigger this event. 
         */
        public OperationPlanEvent(Operation operation, PlanEventKey key) {
            this.operation = operation;
            this.key = key;
        }

        @Override
        public GuiEventKey getKey() {
            return key;
        }

        @Override
        public Operation getContent() {
            return operation;
        }
    }
    
     private class PlanningEventProducer extends BaseGuiEventProducer{

        public PlanningEventProducer() {
            super(PlanEventKey.class);
        }

    }
}
